from .Ansi import ANSI


def log(
    *values: object,
    sep: str | None = " ",
    end: str | None = f"{ANSI.RESET}\n",
):
    print(*values, sep=sep, end=end)


def info(
    *values: object,
    sep: str | None = " ",
    end: str | None = f"{ANSI.RESET}\n",
):
    print(ANSI.CYAN, sep="", end="")
    print(*values, sep=sep, end=end)


def error(
    *values: object,
    sep: str | None = " ",
    end: str | None = f"{ANSI.RESET}\n",
):
    print(ANSI.RED, sep="", end="")
    print(*values, sep=sep, end=end)


def ex(
    value: Exception,
    sep: str | None = " ",
    end: str | None = f"{ANSI.RESET}\n",
):
    print(ANSI.RED, sep="", end="")
    print(f"[{type(value)}] {value}", sep=sep, end=end)


def debug(
    *values: object,
    sep: str | None = " ",
    end: str | None = f"{ANSI.RESET}\n",
):
    print(ANSI.GREEN, sep="", end="")
    print(*values, sep=sep, end=end)



import os
import sys
import console


# -------------------------------------------------------------------------------------------- #
# Contenuto che dovrà essere scritto nel service file
CONTENT = """[Unit]
Description={name} service
After=multi-user.target

[Service]
Type=simple
ExecStart=/usr/bin/python3 {path}
Restart=always
RuntimeMaxSec=infinity

[Install]
WantedBy=multi-user.target"""


# -------------------------------------------------------------------------------------------- #
def main() -> int:
    try:
        # Accetto solo 2 parametri:
        #   0. Nome del file che python eseguire
        #   1. Il mio parametro, ovvero il file che dovrà essere convertito in Service
        if len(sys.argv) != 2:
            raise Exception(
                "Fornire il path dello script che si vuole trasformare in Service!"
            )

        # Path Relativo -> MyProject/MyScript.py
        relative_script_path = sys.argv[1]
        # Path Absolute -> /home/<User>/Desktop/MyProject/MyScript.py
        abs_script_path = os.path.abspath(relative_script_path)
        # Path Dir -> /home/<User>/Desktop/MyProject
        dir_path = os.path.dirname(abs_script_path)
        # Full Name -> MyScript.py
        full_name_script_path = os.path.basename(relative_script_path)
        # Name -> MyScript
        name_script_path = ".".join(full_name_script_path.split(".")[:-1])

        # Check if Script is a File
        if not os.path.isfile(abs_script_path):
            raise Exception("Fornire il path di uno script .py!")

        # Creo un file momentaneo
        temp_file_path = os.path.join(dir_path, f"{name_script_path}.service")
        try:
            with open(temp_file_path, "w") as tmp_file:
                tmp_file.write(
                    CONTENT.format(name=name_script_path, path=abs_script_path)
                )

            # Controllo se il path finale esiste
            service_path = "/etc/systemd/system/"
            if not os.path.exists(service_path):
                raise Exception(f"Impossibile trovare la directory: '{service_path}'")

            # Copio il temp file nella folder dei servizi
            os.system(f"sudo cp {temp_file_path} {service_path}")
        finally:
            # Elimino il temp_file
            if os.path.exists(temp_file_path):
                os.remove(temp_file_path)

        # Controllo che mi trovo su Linux
        if sys.platform != "linux" and sys.platform != "linux2":
            raise Exception(
                "Sistema operativo incompatibile, deve essere linux o linux2"
            )

        # Enable Servizio & Start IT
        os.system("sudo systemctl daemon-reload")
        os.system(f"sudo systemctl enable {script_name}")
        os.system(f"sudo systemctl start {script_name}")
        os.system(f"sudo systemctl status {script_name}")

    except Exception as ex:
        console.error(ex)
        return -1

    return 0



from machine import Pin, SPI
import time

from .board import Board as IBoard


class Board(IBoard):
    # --- Pins --- #
    nss: Pin

    def __init__(self):
        super().__init__(
            8,
            19,
            16,
            18,
            9,
            7,
            10,
            None,
            None,
            None,
            None,
        )

        # Creo il pin NSS
        self.nss = self.pin(self.pin_nss, in_out=Pin.OUT)

    # -------------------------------------------------------------------------------------------- #
    def spi(self) -> object:
        spi = SPI(
            0,
            baudrate=10_000_000,
            polarity=0,
            phase=0,
            sck=Pin(self.pin_sck, Pin.OUT, Pin.PULL_DOWN),
            mosi=Pin(self.pin_mosi, Pin.OUT, Pin.PULL_UP),
            miso=Pin(self.pin_miso, Pin.IN, Pin.PULL_UP),
        )
        spi.init()

        return spi

    def pin(self, pin_id: int | None, **args: object) -> object:
        return Pin(pin_id, args["in_out"])

    def add_event(self, pin_id: int | None, callback: object) -> None:
        if pin_id is not None:
            pin = self.pin(pin_id, in_out=Pin.IN)
            pin.irq(handler=callback, trigger=Pin.IRQ_RISING)

    # -------------------------------------------------------------------------------------------- #
    def reset(self) -> None:
        rst = self.pin(self.pin_rst, in_out=Pin.OUT)  # Set Pin in OUTPUT mode
        rst.off()  # Send low

        time.sleep(0.2)
        rst.on()  # Send High
        time.sleep(0.2)

    def cleanup(self) -> None:
        pass  # Non esiste un equivalente di cleanup per la libreria machine

    # -------------------------------------------------------------------------------------------- #
    def _write(self, reg: int, value: list[int]) -> None:
        wb = bytes([reg | 0x80])
        try:
            self.nss.value(0)
            self._spi.write(wb + bytes(value))
        finally:
            self.nss.value(1)

    def read(self, reg: int) -> int:
        try:
            self.nss.value(0)
            return self._spi.read(2, reg)[1]
        finally:
            self.nss.value(1)

    def reads(self, reg: int, length: int) -> list[int]:
        try:
            self.nss.value(0)
            return self._spi.read(length + 1, reg)[1:]
        finally:
            self.nss.value(1)
